package hr.fer.tel.ruazosa.lectures;

import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.View;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;

import java.util.List;

import hr.fer.tel.ruazosa.lectures.entity.ShortCourse;
import hr.fer.tel.ruazosa.lectures.net.RestFactory;
import hr.fer.tel.ruazosa.lectures.net.RestInterface;

public class MainActivity extends AppCompatActivity {

    private ListView listView;
    private Button loadCourseButton;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.fab);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Snackbar.make(view, "Replace with your own action", Snackbar.LENGTH_LONG)
                        .setAction("Action", null).show();
            }
        });

        listView = (ListView) findViewById(R.id.listView);
        listView.setOnItemClickListener(
            new AdapterView.OnItemClickListener() {

                @Override
                public void onItemClick(AdapterView<?> parent, final View view,
                                        int position, long id) {
                    Object itemAtPosition = parent.getItemAtPosition(position);
                    ShortCourse shortCourse = (ShortCourse) itemAtPosition;
                    Intent intent = new Intent(MainActivity.this, CourseDetailsActivity.class);
                    intent.putExtra("course", shortCourse);
                    startActivity(intent);
                }
            });


        loadCourseButton = (Button) findViewById(R.id.loadCourseButton);
        loadCourseButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                AsyncTask<Void, Void, List<ShortCourse>> task = new AsyncTask<Void, Void, List<ShortCourse>>() {
                    @Override
                    protected List<ShortCourse> doInBackground(Void... params) {
                        RestInterface rest = RestFactory.getInstance();

                        return rest.getListOfCourses();
                    }

                    @Override
                    protected void onPostExecute(List<ShortCourse> courses) {
                        updateCourseList(courses);
                    }
                };
                task.execute();

            }
        });
    }

    private void updateCourseList(List<ShortCourse> courses) {
        Log.d("RUAZOSA", "courses isNull = " + (courses == null));
        CourseAdapter adapter = new CourseAdapter(this,
                android.R.layout.simple_list_item_1, courses);
        listView.setAdapter(adapter);
    }

    private class CourseAdapter extends ArrayAdapter<ShortCourse> {
        private List<ShortCourse> shortCourseList;

        public CourseAdapter(Context context, int textViewResourceId, List<ShortCourse> courses) {
            super(context, textViewResourceId, courses);
            shortCourseList = courses;
        }
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }
}
