package hr.fer.tel.ruazosa.lectures.net.inmemory;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import hr.fer.tel.ruazosa.lectures.entity.Course;
import hr.fer.tel.ruazosa.lectures.entity.Person;
import hr.fer.tel.ruazosa.lectures.entity.ShortCourse;
import hr.fer.tel.ruazosa.lectures.entity.ShortPerson;
import hr.fer.tel.ruazosa.lectures.net.RestInterface;

public class InMemoryRestImplementation implements RestInterface {
    private List<Person> persons;
    private List<Course> courses;
    private Map<Long, List<Person>> enrolled; // CourseId, list of students

    public InMemoryRestImplementation() {
        persons = new LinkedList<>();
        courses = new LinkedList<>();
        enrolled = new HashMap<>();

        Course c = new Course(1L, "RUAZOSA", "Razvoj usluga i aplikacija za operacijski sustav Android");
        courses.add(c);

        c = new Course(2L, "KP", "Konkurentno programiranje");
        courses.add(c);

        c = new Course(3L, "RASSUS", "Raspodijeljeni sustavi");
        courses.add(c);

        c = new Course(4L, "OOP", "Objektno orijentirano programiranje");
        courses.add(c);

        Person p = new Person(1L, "Pero", "Perić", "1111", "C15-15");
        persons.add(p);
        c = getCourse(1L);
        c.setTeacher(new ShortPerson(p));

        p = new Person(2L, "Iva", "Ivić", "555", "C0-32");
        persons.add(p);
        enrollPersonToCourse(p.getId(), c.getId());

        p = new Person(3L, "Jura", "Jurić", "333", "C3-81");
        persons.add(p);
        enrollPersonToCourse(p.getId(), c.getId());
    }

    @Override
    public List<ShortCourse> getListOfCourses() {
        List<ShortCourse> result = new LinkedList<>();
        for(Course c: courses)
            result.add(new ShortCourse(c));

        return result;
    }

    @Override
    public Course getCourse(Long id) {
        for(Course c: courses)
            if(c.getId().equals(id)) return c;
        return null;
    }

    @Override
    public List<ShortPerson> getCourseStudents(Long courseId) {
        List<ShortPerson> result = new LinkedList<>();
        List<Person> enrolledPersonList = enrolled.get(courseId);
        if(enrolledPersonList != null) {
            for (Person p : enrolledPersonList)
                result.add(new ShortPerson(p));
        }

        return result;
    }

    @Override
    public List<ShortPerson> getListOfPersons() {
        List<ShortPerson> result = new LinkedList<>();
        for(Person p: persons)
            result.add(new ShortPerson(p));
        return result;
    }

    @Override
    public Person getPerson(Long id) {
        for(Person p: persons)
            if(p.getId().equals(id)) return p;
        return null;
    }

    @Override
    public boolean enrollPersonToCourse(Long personId, Long courseId) {
        List<Person> peronsEnrolled = enrolled.get(courseId);
        if(peronsEnrolled == null) {
            peronsEnrolled = new LinkedList<>();
            enrolled.put(courseId, peronsEnrolled);
        }
        peronsEnrolled.add(getPerson(personId));
        return true;
    }

    @Override
    public boolean unenrollPersonFromCourse(Long personId, Long courseId) {
        List<Person> peronsEnrolled = enrolled.get(courseId);
        if(peronsEnrolled == null)
            return true;

        for(Person p: peronsEnrolled) {
            if(p.getId().equals(personId))
                peronsEnrolled.remove(p);
        }
        return true;
    }
}
