package hr.fer.tel.ruazosa.lectures.net.resttemplate;

import android.support.annotation.NonNull;
import android.util.Base64;
import android.util.Log;

import org.springframework.http.HttpRequest;
import org.springframework.http.client.ClientHttpRequestExecution;
import org.springframework.http.client.ClientHttpRequestFactory;
import org.springframework.http.client.ClientHttpRequestInterceptor;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.LinkedList;
import java.util.List;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import hr.fer.tel.ruazosa.lectures.entity.Course;
import hr.fer.tel.ruazosa.lectures.entity.Person;
import hr.fer.tel.ruazosa.lectures.entity.ShortCourse;
import hr.fer.tel.ruazosa.lectures.entity.ShortPerson;
import hr.fer.tel.ruazosa.lectures.net.RestFactory;
import hr.fer.tel.ruazosa.lectures.net.RestInterface;

public class HttpsRestSpringTemplete implements RestInterface {
    private final String baseURL;
    private final String username;
    private final String password;

    public HttpsRestSpringTemplete(String username, String password) {
        this.username = username;
        this.password = password;

        baseURL = "https://" + RestFactory.BASE_IP + ":8443/api";

        trustAll();
    }

    private void trustAll() {
        TrustManager[] trustAllCerts = new TrustManager[] {new X509TrustManager() {
            public java.security.cert.X509Certificate[] getAcceptedIssuers() {
                return new java.security.cert.X509Certificate[] {};
            }

            public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
            }

            public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
            }
        } };

        try {
            SSLContext sc = SSLContext.getInstance("TLS");
            sc.init(null, trustAllCerts, new java.security.SecureRandom());
            HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
        } catch (Exception e) {
            e.printStackTrace();
        }

        HttpsURLConnection.setDefaultHostnameVerifier(new HostnameVerifier() {
            public boolean verify(String hostname, SSLSession session) {
                return true;
            }
        });
    }

    @Override
    public List<ShortCourse> getListOfCourses() {
        try {
            RestTemplate restTemplate = getRestTemplate();

            List<ShortCourse> courseList = restTemplate.getForObject(baseURL + "/courses", ShortCourseList.class);

            return courseList;
        } catch (Exception e) {
            Log.i("RUAZOSA", e.getMessage(), e);
        }

        return null;
    }

    public static class ShortCourseList extends LinkedList<ShortCourse> {}

    @Override
    public Course getCourse(Long id) {
        try {
            RestTemplate restTemplate = getRestTemplate();

            Course course = restTemplate.getForObject(baseURL + "/courses/" + id, Course.class);

            return course;
        } catch (Exception e) {
            Log.i("RUAZOSA", e.getMessage(), e);
        }

        return null;
    }

    @Override
    public List<ShortPerson> getCourseStudents(Long courseId) {
        try {
            RestTemplate restTemplate = getRestTemplate();

            List<ShortPerson> personList = restTemplate.getForObject(baseURL + "/courses/" + courseId + "/students", ShortPersonList.class);

            return personList;
        } catch (Exception e) {
            Log.i("RUAZOSA", e.getMessage(), e);
        }

        return null;
    }

    public static class ShortPersonList extends LinkedList<ShortPerson> {}

    @Override
    public List<ShortPerson> getListOfPersons() {
        throw new UnsupportedOperationException();
    }

    @Override
    public Person getPerson(Long id) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean enrollPersonToCourse(Long personId, Long courseId) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean unenrollPersonFromCourse(Long personId, Long courseId) {
        throw new UnsupportedOperationException();
    }

    @NonNull
    private RestTemplate getRestTemplate() {
        RestTemplate restTemplate = new RestTemplate();
        List<ClientHttpRequestInterceptor> interceptors = restTemplate.getInterceptors();
        if(interceptors == null)
            interceptors = new LinkedList<>();
        else {
            List<ClientHttpRequestInterceptor> tempList = new LinkedList<>();
            tempList.addAll(interceptors);
            interceptors = tempList;
        }

        interceptors.add(new ClientHttpRequestInterceptor() {
            @Override
            public ClientHttpResponse intercept(HttpRequest request, byte[] body, ClientHttpRequestExecution execution) throws IOException {
                String authorization = username + ":" + password;
                byte[] encodedBytes;
                encodedBytes = Base64.encode(authorization.getBytes(), 0);
                authorization = "Basic " + new String(encodedBytes);
                Log.d("RUAZOSA", authorization);
                request.getHeaders().add("Authorization", authorization);
                return execution.execute(request, body);
            }
        });
        restTemplate.setInterceptors(interceptors);

        restTemplate.getMessageConverters().add(new MappingJackson2HttpMessageConverter());
        return restTemplate;
    }


}
