package hr.fer.tel.ruazosa.lectures.net.retrofit;

import android.util.Base64;
import android.util.Log;

import com.squareup.okhttp.OkHttpClient;

import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.List;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import hr.fer.tel.ruazosa.lectures.entity.Course;
import hr.fer.tel.ruazosa.lectures.entity.Person;
import hr.fer.tel.ruazosa.lectures.entity.ShortCourse;
import hr.fer.tel.ruazosa.lectures.entity.ShortPerson;
import hr.fer.tel.ruazosa.lectures.net.RestFactory;
import hr.fer.tel.ruazosa.lectures.net.RestInterface;
import retrofit.RequestInterceptor;
import retrofit.RestAdapter;
import retrofit.client.OkClient;

public class HttpsRestRetrofit implements RestInterface {
    private final String baseURL;
    private final String username;
    private final String password;
    private final LecturesService service;


    public HttpsRestRetrofit(String username, String password) {
        this.username = username;
        this.password = password;

        baseURL = "https://" + RestFactory.BASE_IP + ":8443/api";
        RestAdapter retrofit = new RestAdapter.Builder()
                .setEndpoint(baseURL)
                .setClient(new OkClient(configureClient(new OkHttpClient())))
                .setRequestInterceptor(getAuthInterceptor())
                .build();

        service = retrofit.create(LecturesService.class);
    }

    private RequestInterceptor getAuthInterceptor() {
        return new RequestInterceptor() {
            @Override
            public void intercept(RequestFacade request) {
                String authorization = username + ":" + password;
                authorization = "Basic " + Base64.encodeToString(authorization.getBytes(), Base64.NO_WRAP);
                Log.d("RUAZOSA", authorization);
                request.addHeader("Authorization", authorization);
            }
        };
    }

    public static OkHttpClient configureClient(final OkHttpClient client) {
        final TrustManager[] certs = new TrustManager[]{new X509TrustManager() {

            @Override
            public X509Certificate[] getAcceptedIssuers() {
                return null;
            }

            @Override
            public void checkServerTrusted(final X509Certificate[] chain,
                                           final String authType) throws CertificateException {
            }

            @Override
            public void checkClientTrusted(final X509Certificate[] chain,
                                           final String authType) throws CertificateException {
            }
        }};

        SSLContext ctx = null;
        try {
            ctx = SSLContext.getInstance("TLS");
            ctx.init(null, certs, new SecureRandom());
        } catch (final java.security.GeneralSecurityException ex) {
        }

        try {
            final HostnameVerifier hostnameVerifier = new HostnameVerifier() {
                @Override
                public boolean verify(final String hostname,
                                      final SSLSession session) {
                    return true;
                }
            };
            client.setHostnameVerifier(hostnameVerifier);
            client.setSslSocketFactory(ctx.getSocketFactory());
        } catch (final Exception e) {
        }

        return client;
    }

    @Override
    public List<ShortCourse> getListOfCourses() {
        return service.getListOfCourses();
    }

    @Override
    public Course getCourse(Long id) {
        return service.getCourse(id);
    }

    @Override
    public List<ShortPerson> getCourseStudents(Long courseId) {
        return service.getCourseStudents(courseId);
    }

    @Override
    public List<ShortPerson> getListOfPersons() {
        throw new UnsupportedOperationException();
    }

    @Override
    public Person getPerson(Long id) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean enrollPersonToCourse(Long personId, Long courseId) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean unenrollPersonFromCourse(Long personId, Long courseId) {
        throw new UnsupportedOperationException();
    }
}
