package hr.fer.tel.ruazosa.lectures.net.urlconnection;

import android.util.Base64;
import android.util.Log;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.LinkedList;
import java.util.List;


import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import hr.fer.tel.ruazosa.lectures.entity.Course;
import hr.fer.tel.ruazosa.lectures.entity.Person;
import hr.fer.tel.ruazosa.lectures.entity.ShortCourse;
import hr.fer.tel.ruazosa.lectures.entity.ShortPerson;
import hr.fer.tel.ruazosa.lectures.net.RestFactory;
import hr.fer.tel.ruazosa.lectures.net.RestInterface;

public class HttpsRestUrlConnection implements RestInterface {
    private final String baseURL;
    private final String userName;
    private final String password;
    private static final HostnameVerifier DO_NOT_VERIFY = new HostnameVerifier() {
        public boolean verify(String hostname, SSLSession session) {
            return true;
        }
    };

    public HttpsRestUrlConnection(String userName, String password) {
        this.userName = userName;
        this.password = password;
        baseURL = "https://" + RestFactory.BASE_IP + ":8443/api";

        trustAll();
    }

    private void trustAll() {
        TrustManager[] trustAllCerts = new TrustManager[] {new X509TrustManager() {
            public java.security.cert.X509Certificate[] getAcceptedIssuers() {
                    return new java.security.cert.X509Certificate[] {};
                }

                public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                }

                public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                }
            }
        };

        try {
            SSLContext sc = SSLContext.getInstance("TLS");
            sc.init(null, trustAllCerts, new java.security.SecureRandom());
            HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    @Override
    public List<ShortCourse> getListOfCourses() {
        try {
            String textURL = baseURL + "/courses";
            String coursesText = loadData(textURL);
            Log.d("RUAZOSA", "GET " + textURL + " : Response: " + coursesText);

            JSONArray arrayOfCourses = new JSONArray(coursesText);

            List<ShortCourse> list = new LinkedList<>();
            for (int i = 0; i < arrayOfCourses.length(); i++) {
                ShortCourse c = parseShortCourse((JSONObject)arrayOfCourses.get(i));
                list.add(c);
            }

            return list;
        } catch (JSONException e) {
            Log.d("RUAZOSA", "problem parsing json", e);
            return new LinkedList<>();
        }
    }

    @Override
    public Course getCourse(Long id) {
        try {
            String textURL = baseURL + "/courses/" + id;
            String courseText = loadData(textURL);
            Log.d("RUAZOSA", "GET " + textURL + " : Response: " + courseText);

            JSONObject jsonCourse = new JSONObject(courseText);
            Course course = new Course();
            course.setId(jsonCourse.getLong("id"));
            course.setName(jsonCourse.getString("name"));
            if(!jsonCourse.isNull("teacher"))
                course.setTeacher(parseShortPerson(jsonCourse.getJSONObject("teacher")));
            course.setDescription(jsonCourse.getString("description"));

            return course;
        } catch (JSONException e) {
            Log.d("RUAZOSA", "problem parsing json", e);
            return null;
        }
    }

    @Override
    public List<ShortPerson> getCourseStudents(Long courseId) {
        try {
            String textURL = baseURL + "/courses/" + courseId + "/students";
            String studentsText = loadData(textURL);
            Log.d("RUAZOSA", "GET " + textURL + " : Response: " + studentsText);
            JSONArray jsonStudents = new JSONArray(studentsText);

            List<ShortPerson> list = new LinkedList<>();
            for (int i = 0; i < jsonStudents.length(); i++) {
                JSONObject jsonStudent = (JSONObject) jsonStudents.get(i);
                ShortPerson student = parseShortPerson(jsonStudent);
                list.add(student);
            }

            return list;
        } catch (JSONException e) {
            Log.d("RUAZOSA", "problem parsing json", e);
            return new LinkedList<>();
        }
    }

    @Override
    public List<ShortPerson> getListOfPersons() {
        throw new UnsupportedOperationException();
    }

    @Override
    public Person getPerson(Long id) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean enrollPersonToCourse(Long personId, Long courseId) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean unenrollPersonFromCourse(Long personId, Long courseId) {
        throw new UnsupportedOperationException();
    }

    private ShortPerson parseShortPerson(JSONObject jsonPerson) throws JSONException {
        ShortPerson person = new ShortPerson();
        person.setId(jsonPerson.getLong("id"));
        person.setName(jsonPerson.getString("name"));
        return person;
    }

    private ShortCourse parseShortCourse(JSONObject jsonShortCourse) throws JSONException {
        ShortCourse course = new ShortCourse();
        course.setId(jsonShortCourse.getLong("id"));
        course.setName(jsonShortCourse.getString("name"));
        return course;
    }

    private String loadData(String textURL) {
        try {
            URL url = new URL(textURL);
            // HTTPS
            HttpsURLConnection connection = (HttpsURLConnection) url.openConnection();
            connection.setHostnameVerifier(DO_NOT_VERIFY);

            // AUTH
            String authorization = userName + ":" + password;
            byte[] encodedBytes;
            encodedBytes = Base64.encode(authorization.getBytes(), 0);
            authorization = "Basic " + new String(encodedBytes);
            Log.d("RUAZOSA", authorization);
            connection.setRequestProperty("Authorization", authorization);

            connection.setRequestMethod("GET");
            connection.setRequestProperty("Accept", "application/json");
            if (connection.getResponseCode() != 200) {
                return "" + connection.getResponseCode();
            }

            InputStream contentStream = connection.getInputStream();
            String text = readTextFromStream(contentStream);
            connection.disconnect();
            Log.d("RUAZOSA", "učitao sam: " + text);
            return text;
        } catch (MalformedURLException e) {
            Log.i("RUAZOSA", "Try to load: " + textURL, e);
        } catch (ProtocolException e) {
            Log.i("RUAZOSA", "Try to load: " + textURL, e);
        } catch (IOException e) {
            Log.i("RUAZOSA", "Try to load: " + textURL, e);
        }
        return null;
    }

    private String readTextFromStream(InputStream contentStream)
            throws IOException {
        BufferedReader reader = new BufferedReader(
                new InputStreamReader(contentStream));

        StringBuffer sb = new StringBuffer();
        String line = reader.readLine();
        while (line != null) {
            sb.append(line);
            sb.append("\n");
            line = reader.readLine();
        }
        String text = sb.toString();
        return text;
    }
}
