package hr.fer.tel.ruazosa.lectures.net.urlconnection;

import android.util.Log;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;
import java.util.LinkedList;
import java.util.List;

import hr.fer.tel.ruazosa.lectures.entity.Course;
import hr.fer.tel.ruazosa.lectures.entity.Person;
import hr.fer.tel.ruazosa.lectures.entity.ShortCourse;
import hr.fer.tel.ruazosa.lectures.entity.ShortPerson;
import hr.fer.tel.ruazosa.lectures.net.RestFactory;
import hr.fer.tel.ruazosa.lectures.net.RestInterface;

public class RestUrlConnection implements RestInterface {
    private final String baseURL;

    public RestUrlConnection() {
        baseURL = "http://" + RestFactory.BASE_IP + ":8080/api";
    }

    @Override
    public List<ShortCourse> getListOfCourses() {
        try {
            String coursesText = loadData(baseURL + "/courses");
            JSONArray arrayOfCourses = new JSONArray(coursesText);

            List<ShortCourse> list = new LinkedList<>();
            for (int i = 0; i < arrayOfCourses.length(); i++) {
                ShortCourse c = parseShortCourse((JSONObject)arrayOfCourses.get(i));
                list.add(c);
            }

            return list;
        } catch (JSONException e) {
            Log.d("RUAZOSA", "problem parsing json", e);
            return new LinkedList<>();
        }
    }

    @Override
    public Course getCourse(Long id) {
        try {
            String courseText = loadData(baseURL + "/courses/" + id);
            JSONObject jsonCourse = new JSONObject(courseText);
            Course course = new Course();
            course.setId(jsonCourse.getLong("id"));
            course.setName(jsonCourse.getString("name"));
            if(!jsonCourse.isNull("teacher"))
                course.setTeacher(parseShortPerson(jsonCourse.getJSONObject("teacher")));
            course.setDescription(jsonCourse.getString("description"));

            return course;
        } catch (JSONException e) {
            Log.d("RUAZOSA", "problem parsing json", e);
            return null;
        }
    }

    @Override
    public List<ShortPerson> getCourseStudents(Long courseId) {
        try {
            String studentsText = loadData(baseURL + "/courses/" + courseId + "/students");
            JSONArray jsonStudents = new JSONArray(studentsText);

            List<ShortPerson> list = new LinkedList<>();
            for (int i = 0; i < jsonStudents.length(); i++) {
                JSONObject jsonStudent = (JSONObject) jsonStudents.get(i);
                ShortPerson student = parseShortPerson(jsonStudent);
                list.add(student);
            }

            return list;
        } catch (JSONException e) {
            Log.d("RUAZOSA", "problem parsing json", e);
            return new LinkedList<>();
        }
    }

    @Override
    public List<ShortPerson> getListOfPersons() {
        throw new UnsupportedOperationException();
    }

    @Override
    public Person getPerson(Long id) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean enrollPersonToCourse(Long personId, Long courseId) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean unenrollPersonFromCourse(Long personId, Long courseId) {
        throw new UnsupportedOperationException();
    }

    private ShortPerson parseShortPerson(JSONObject jsonPerson) throws JSONException {
        ShortPerson person = new ShortPerson();
        person.setId(jsonPerson.getLong("id"));
        person.setName(jsonPerson.getString("name"));
        return person;
    }

    private ShortCourse parseShortCourse(JSONObject jsonShortCourse) throws JSONException {
        ShortCourse course = new ShortCourse();
        course.setId(jsonShortCourse.getLong("id"));
        course.setName(jsonShortCourse.getString("name"));
        return course;
    }

    private String loadData(String textURL) {
        try {
            URL url = new URL(textURL);
            HttpURLConnection connection = (HttpURLConnection) url.openConnection();
            connection.setRequestMethod("GET");
            connection.setRequestProperty("Accept", "application/json");
            if (connection.getResponseCode() != 200) {
                return "" + connection.getResponseCode();
            }

            InputStream contentStream = connection.getInputStream();
            String text = readTextFromStream(contentStream);
            connection.disconnect();
            Log.d("RUAZOSA", "učitao sam: " + text);
            return text;
        } catch (MalformedURLException e) {
            Log.i("RUAZOSA", "Try to load: " + textURL, e);
        } catch (ProtocolException e) {
            Log.i("RUAZOSA", "Try to load: " + textURL, e);
        } catch (IOException e) {
            Log.i("RUAZOSA", "Try to load: " + textURL, e);
        }
        return null;
    }

    private String readTextFromStream(InputStream contentStream)
            throws IOException {
        BufferedReader reader = new BufferedReader(
                new InputStreamReader(contentStream));

        StringBuffer sb = new StringBuffer();
        String line = reader.readLine();
        while (line != null) {
            sb.append(line);
            sb.append("\n");
            line = reader.readLine();
        }
        String text = sb.toString();
        return text;
    }
}
